@@ -70,16 +70,16 @@ class stream_engine_t : public io_object_t, public i_engine
 
     stream_engine_t (fd_t fd_,
                      const options_t &options_,
-                     const std::string &endpoint_);
+                     const endpoint_uri_pair_t &endpoint_uri_pair_);
     ~stream_engine_t ();
 
     //  i_engine interface implementation.
     void plug (zmq::io_thread_t *io_thread_, zmq::session_base_t *session_);
     void terminate ();
-    void restart_input ();
+    bool restart_input ();
     void restart_output ();
     void zap_msg_available ();
-    const char *get_endpoint () const;
+    const endpoint_uri_pair_t &get_endpoint () const;
 
     //  i_poll_events interface implementation.
     void in_event ();
@@ -93,12 +93,22 @@ class stream_engine_t : public io_object_t, public i_engine
     //  Function to handle network disconnections.
     void error (error_reason_t reason_);
 
-    //  Receives the greeting message from the peer.
-    int receive_greeting ();
-
     //  Detects the protocol used by the peer.
     bool handshake ();
 
+    //  Receive the greeting from the peer.
+    int receive_greeting ();
+    void receive_greeting_versioned ();
+
+    typedef bool (stream_engine_t::*handshake_fun_t) ();
+    static handshake_fun_t select_handshake_fun (bool unversioned,
+                                                 unsigned char revision);
+
+    bool handshake_v1_0_unversioned ();
+    bool handshake_v1_0 ();
+    bool handshake_v2_0 ();
+    bool handshake_v3_0 ();
+
     int routing_id_msg (msg_t *msg_);
     int process_routing_id_msg (msg_t *msg_);
 
@@ -135,9 +145,6 @@ class stream_engine_t : public io_object_t, public i_engine
     //  Underlying socket.
     fd_t _s;
 
-    //  True iff this is server's engine.
-    bool _as_server;
-
     msg_t _tx_msg;
     //  Need to store PING payload for PONG
     msg_t _pong_msg;
@@ -183,8 +190,8 @@ class stream_engine_t : public io_object_t, public i_engine
 
     const options_t _options;
 
-    // String representation of endpoint
-    std::string _endpoint;
+    //  Representation of the connected endpoints.
+    const endpoint_uri_pair_t _endpoint_uri_pair;
 
     bool _plugged;
 
@@ -231,7 +238,7 @@ class stream_engine_t : public io_object_t, public i_engine
     // Socket
     zmq::socket_base_t *_socket;
 
-    std::string _peer_address;
+    const std::string _peer_address;
 
     stream_engine_t (const stream_engine_t &);
     const stream_engine_t &operator= (const stream_engine_t &);
