@@ -35,7 +35,6 @@
 #include "tcp_connecter.hpp"
 #include "stream_engine.hpp"
 #include "io_thread.hpp"
-#include "random.hpp"
 #include "err.hpp"
 #include "ip.hpp"
 #include "tcp.hpp"
@@ -69,40 +68,16 @@ zmq::tcp_connecter_t::tcp_connecter_t (class io_thread_t *io_thread_,
                                        const options_t &options_,
                                        address_t *addr_,
                                        bool delayed_start_) :
-    own_t (io_thread_, options_),
-    io_object_t (io_thread_),
-    _addr (addr_),
-    _s (retired_fd),
-    _handle (static_cast<handle_t> (NULL)),
-    _delayed_start (delayed_start_),
-    _connect_timer_started (false),
-    _reconnect_timer_started (false),
-    _session (session_),
-    _current_reconnect_ivl (options.reconnect_ivl),
-    _socket (_session->get_socket ())
+    stream_connecter_base_t (
+      io_thread_, session_, options_, addr_, delayed_start_),
+    _connect_timer_started (false)
 {
-    zmq_assert (_addr);
-    zmq_assert (_addr->protocol == "tcp");
-    _addr->to_string (_endpoint);
-    // TODO the return value is unused! what if it fails? if this is impossible
-    // or does not matter, change such that endpoint in initialized using an
-    // initializer, and make endpoint const
+    zmq_assert (_addr->protocol == protocol_name::tcp);
 }
 
 zmq::tcp_connecter_t::~tcp_connecter_t ()
 {
     zmq_assert (!_connect_timer_started);
-    zmq_assert (!_reconnect_timer_started);
-    zmq_assert (!_handle);
-    zmq_assert (_s == retired_fd);
-}
-
-void zmq::tcp_connecter_t::process_plug ()
-{
-    if (_delayed_start)
-        add_reconnect_timer ();
-    else
-        start_connecting ();
 }
 
 void zmq::tcp_connecter_t::process_term (int linger_)
@@ -112,27 +87,7 @@ void zmq::tcp_connecter_t::process_term (int linger_)
         _connect_timer_started = false;
     }
 
-    if (_reconnect_timer_started) {
-        cancel_timer (reconnect_timer_id);
-        _reconnect_timer_started = false;
-    }
-
-    if (_handle) {
-        rm_handle ();
-    }
-
-    if (_s != retired_fd)
-        close ();
-
-    own_t::process_term (linger_);
-}
-
-void zmq::tcp_connecter_t::in_event ()
-{
-    //  We are not polling for incoming data, so we are actually called
-    //  because of error here. However, we can get error on out event as well
-    //  on some platforms, so we'll simply handle both events in the same way.
-    out_event ();
+    stream_connecter_base_t::process_term (linger_);
 }
 
 void zmq::tcp_connecter_t::out_event ()
@@ -142,6 +97,9 @@ void zmq::tcp_connecter_t::out_event ()
         _connect_timer_started = false;
     }
 
+    //  TODO this is still very similar to (t)ipc_connecter_t, maybe the
+    //  differences can be factored out
+
     rm_handle ();
 
     const fd_t fd = connect ();
@@ -153,38 +111,18 @@ void zmq::tcp_connecter_t::out_event ()
         return;
     }
 
-    //  Create the engine object for this connection.
-    stream_engine_t *engine =
-      new (std::nothrow) stream_engine_t (fd, options, _endpoint);
-    alloc_assert (engine);
-
-    //  Attach the engine to the corresponding session object.
-    send_attach (_session, engine);
-
-    //  Shut the connecter down.
-    terminate ();
-
-    _socket->event_connected (_endpoint, fd);
-}
-
-void zmq::tcp_connecter_t::rm_handle ()
-{
-    rm_fd (_handle);
-    _handle = static_cast<handle_t> (NULL);
+    create_engine (fd, get_socket_name<tcp_address_t> (fd, socket_end_local));
 }
 
 void zmq::tcp_connecter_t::timer_event (int id_)
 {
-    zmq_assert (id_ == reconnect_timer_id || id_ == connect_timer_id);
     if (id_ == connect_timer_id) {
         _connect_timer_started = false;
         rm_handle ();
         close ();
         add_reconnect_timer ();
-    } else if (id_ == reconnect_timer_id) {
-        _reconnect_timer_started = false;
-        start_connecting ();
-    }
+    } else
+        stream_connecter_base_t::timer_event (id_);
 }
 
 void zmq::tcp_connecter_t::start_connecting ()
@@ -202,7 +140,8 @@ void zmq::tcp_connecter_t::start_connecting ()
     else if (rc == -1 && errno == EINPROGRESS) {
         _handle = add_fd (_s);
         set_pollout (_handle);
-        _socket->event_connect_delayed (_endpoint, zmq_errno ());
+        _socket->event_connect_delayed (
+          make_unconnected_connect_endpoint_pair (_endpoint), zmq_errno ());
 
         //  add userspace connect timeout
         add_connect_timer ();
@@ -224,30 +163,6 @@ void zmq::tcp_connecter_t::add_connect_timer ()
     }
 }
 
-void zmq::tcp_connecter_t::add_reconnect_timer ()
-{
-    const int interval = get_new_reconnect_ivl ();
-    add_timer (interval, reconnect_timer_id);
-    _socket->event_connect_retried (_endpoint, interval);
-    _reconnect_timer_started = true;
-}
-
-int zmq::tcp_connecter_t::get_new_reconnect_ivl ()
-{
-    //  The new interval is the current interval + random value.
-    const int interval =
-      _current_reconnect_ivl + generate_random () % options.reconnect_ivl;
-
-    //  Only change the current reconnect interval  if the maximum reconnect
-    //  interval was set and if it's larger than the reconnect interval.
-    if (options.reconnect_ivl_max > 0
-        && options.reconnect_ivl_max > options.reconnect_ivl)
-        //  Calculate the next interval
-        _current_reconnect_ivl =
-          std::min (_current_reconnect_ivl * 2, options.reconnect_ivl_max);
-    return interval;
-}
-
 int zmq::tcp_connecter_t::open ()
 {
     zmq_assert (_s == retired_fd);
@@ -259,63 +174,22 @@ int zmq::tcp_connecter_t::open ()
 
     _addr->resolved.tcp_addr = new (std::nothrow) tcp_address_t ();
     alloc_assert (_addr->resolved.tcp_addr);
-    int rc = _addr->resolved.tcp_addr->resolve (_addr->address.c_str (), false,
-                                                options.ipv6);
-    if (rc != 0) {
+    _s = tcp_open_socket (_addr->address.c_str (), options, false, true,
+                          _addr->resolved.tcp_addr);
+    if (_s == retired_fd) {
+        //  TODO we should emit some event in this case!
+
         LIBZMQ_DELETE (_addr->resolved.tcp_addr);
         return -1;
     }
     zmq_assert (_addr->resolved.tcp_addr != NULL);
-    const tcp_address_t *const tcp_addr = _addr->resolved.tcp_addr;
-
-    //  Create the socket.
-    _s = open_socket (tcp_addr->family (), SOCK_STREAM, IPPROTO_TCP);
-
-    //  IPv6 address family not supported, try automatic downgrade to IPv4.
-    if (_s == zmq::retired_fd && tcp_addr->family () == AF_INET6
-        && errno == EAFNOSUPPORT && options.ipv6) {
-        rc = _addr->resolved.tcp_addr->resolve (_addr->address.c_str (), false,
-                                                false);
-        if (rc != 0) {
-            LIBZMQ_DELETE (_addr->resolved.tcp_addr);
-            return -1;
-        }
-        _s = open_socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
-    }
-
-    if (_s == retired_fd) {
-        return -1;
-    }
-
-    //  On some systems, IPv4 mapping in IPv6 sockets is disabled by default.
-    //  Switch it on in such cases.
-    if (tcp_addr->family () == AF_INET6)
-        enable_ipv4_mapping (_s);
-
-    // Set the IP Type-Of-Service priority for this socket
-    if (options.tos != 0)
-        set_ip_type_of_service (_s, options.tos);
-
-    // Bind the socket to a device if applicable
-    if (!options.bound_device.empty ())
-        bind_to_device (_s, options.bound_device);
 
     // Set the socket to non-blocking mode so that we get async connect().
     unblock_socket (_s);
 
-    // Set the socket to loopback fastpath if configured.
-    if (options.loopback_fastpath)
-        tcp_tune_loopback_fast_path (_s);
+    const tcp_address_t *const tcp_addr = _addr->resolved.tcp_addr;
 
-    //  Set the socket buffer limits for the underlying socket.
-    if (options.sndbuf >= 0)
-        set_tcp_send_buffer (_s, options.sndbuf);
-    if (options.rcvbuf >= 0)
-        set_tcp_receive_buffer (_s, options.rcvbuf);
-
-    // Set the IP Type-Of-Service for the underlying socket
-    if (options.tos != 0)
-        set_ip_type_of_service (_s, options.tos);
+    int rc;
 
     // Set a source address for conversations
     if (tcp_addr->has_src_addr ()) {
@@ -428,17 +302,3 @@ bool zmq::tcp_connecter_t::tune_socket (const fd_t fd_)
                    | tune_tcp_maxrt (fd_, options.tcp_maxrt);
     return rc == 0;
 }
-
-void zmq::tcp_connecter_t::close ()
-{
-    zmq_assert (_s != retired_fd);
-#ifdef ZMQ_HAVE_WINDOWS
-    const int rc = closesocket (_s);
-    wsa_assert (rc != SOCKET_ERROR);
-#else
-    const int rc = ::close (_s);
-    errno_assert (rc == 0);
-#endif
-    _socket->event_closed (_endpoint, _s);
-    _s = retired_fd;
-}
