@@ -52,8 +52,9 @@ static unsigned int __stdcall thread_routine (void *arg_)
 }
 }
 
-void zmq::thread_t::start (thread_fn *tfn_, void *arg_)
+void zmq::thread_t::start (thread_fn *tfn_, void *arg_, const char *name_)
 {
+    LIBZMQ_UNUSED (name_);
     _tfn = tfn_;
     _arg = arg_;
 #if defined _WIN32_WCE
@@ -109,8 +110,9 @@ static void *thread_routine (void *arg_)
 }
 }
 
-void zmq::thread_t::start (thread_fn *tfn_, void *arg_)
+void zmq::thread_t::start (thread_fn *tfn_, void *arg_, const char *name_)
 {
+    LIBZMQ_UNUSED (name_);
     _tfn = tfn_;
     _arg = arg_;
     _descriptor = taskSpawn (NULL, DEFAULT_PRIORITY, DEFAULT_OPTIONS,
@@ -179,15 +181,17 @@ static void *thread_routine (void *arg_)
 #endif
     zmq::thread_t *self = (zmq::thread_t *) arg_;
     self->applySchedulingParameters ();
+    self->setThreadName (self->_name.c_str ());
     self->_tfn (self->_arg);
     return NULL;
 }
 }
 
-void zmq::thread_t::start (thread_fn *tfn_, void *arg_)
+void zmq::thread_t::start (thread_fn *tfn_, void *arg_, const char *name_)
 {
     _tfn = tfn_;
     _arg = arg_;
+    _name = name_;
     int rc = pthread_create (&_descriptor, NULL, thread_routine, this);
     posix_assert (rc);
     _started = true;
@@ -228,7 +232,7 @@ void zmq::thread_t::
         return;
     }
 #endif
-    int rc = pthread_getschedparam (_descriptor, &policy, &param);
+    int rc = pthread_getschedparam (pthread_self (), &policy, &param);
     posix_assert (rc);
 
     if (_thread_sched_policy != ZMQ_THREAD_SCHED_POLICY_DFLT) {
@@ -257,7 +261,7 @@ void zmq::thread_t::
         param.sched_priority = -1;
 #endif
 
-    rc = pthread_setschedparam (_descriptor, policy, &param);
+    rc = pthread_setschedparam (pthread_self (), policy, &param);
 
 #if defined(__FreeBSD_kernel__) || defined(__FreeBSD__)
     // If this feature is unavailable at run-time, don't abort.
@@ -309,20 +313,25 @@ void zmq::thread_t::setThreadName (const char *name_)
     if (!name_)
         return;
 
+        /* Fails with permission denied on Android 5/6 */
+#if defined(ZMQ_HAVE_ANDROID)
+    return;
+#endif
+
 #if defined(ZMQ_HAVE_PTHREAD_SETNAME_1)
     int rc = pthread_setname_np (name_);
     if (rc)
         return;
 #elif defined(ZMQ_HAVE_PTHREAD_SETNAME_2)
-    int rc = pthread_setname_np (_descriptor, name_);
+    int rc = pthread_setname_np (pthread_self (), name_);
     if (rc)
         return;
 #elif defined(ZMQ_HAVE_PTHREAD_SETNAME_3)
-    int rc = pthread_setname_np (_descriptor, name_, NULL);
+    int rc = pthread_setname_np (pthread_self (), name_, NULL);
     if (rc)
         return;
 #elif defined(ZMQ_HAVE_PTHREAD_SET_NAME)
-    pthread_set_name_np (_descriptor, name_);
+    pthread_set_name_np (pthread_self (), name_);
 #endif
 }
 
