@@ -16,6 +16,7 @@ use Config;
 # compiler detection
 my $is_gcc = length($Config{gccversion});
 my $is_msvc = $Config{cc} eq 'cl' ? 1 : 0;
+my $legacy_gcc = index ($Config{gccversion}, '4.2.1') != -1 ? 1 : 0;
 
 # os detection
 my $is_solaris = ($^O =~ /(sun|solaris)/i) ? 1 : 0;
@@ -23,6 +24,7 @@ my $is_windows = ($^O =~ /MSWin32/i) ? 1 : 0;
 my $is_linux = ($^O =~ /linux/i) ? 1 : 0;
 my $is_osx = ($^O =~ /darwin/i) ? 1 : 0;
 my $is_bsd = ($^O =~ /bsd/i) ? 1 : 0;
+my $is_openbsd = ($^O =~ /openbsd/i) ? 1 : 0;
 my $is_gkfreebsd = ($^O =~ /gnukfreebsd/i) ? 1 : 0;
 
 my $def = '-DZMQ_CUSTOM_PLATFORM_HPP -DZMQ_STATIC -DZMQ_BUILD_DRAFT_API -D_THREAD_SAFE';
@@ -123,7 +125,7 @@ else
 }
 
 
-if (($is_linux || $is_osx || $is_bsd) && !$is_gkfreebsd)
+if ($is_linux || $is_osx || ($is_bsd && !$is_gkfreebsd && !$is_openbsd))
 {
 	push @opts,
 		'ZMQ_HAVE_TCP_KEEPCNT',
@@ -251,7 +253,7 @@ sub MY::c_o {
 	}
 
 	my $std_switch = '';
-	if ($is_gcc) {
+	if ($is_gcc && !$legacy_gcc) {
 		$std_switch = '-std=c++11'
 	}
 
